package com.objsys.asn1c.cdt;

import java.util.Arrays;
import java.util.List;

import org.eclipse.cdt.core.CCProjectNature;
import org.eclipse.cdt.core.CProjectNature;
import org.eclipse.cdt.core.model.CoreModel;
import org.eclipse.cdt.core.settings.model.ICProjectDescription;
import org.eclipse.cdt.core.settings.model.ICProjectDescriptionManager;
import org.eclipse.cdt.core.settings.model.extension.CConfigurationData;
import org.eclipse.cdt.managedbuilder.core.IBuilder;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.ITargetPlatform;
import org.eclipse.cdt.managedbuilder.core.IToolChain;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.cdt.managedbuilder.internal.core.Configuration;
import org.eclipse.cdt.managedbuilder.internal.core.ManagedBuildInfo;
import org.eclipse.cdt.managedbuilder.internal.core.ManagedProject;
import org.eclipse.cdt.managedbuilder.internal.core.ToolChain;
import org.eclipse.cdt.managedbuilder.ui.wizards.CfgHolder;
import org.eclipse.cdt.ui.wizards.CWizardHandler;
import org.eclipse.cdt.utils.Platform;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;

public class Utility {
   
   public static void configureCProject(IProject project) throws CoreException {
      
      CProjectNature.addCNature(project, new NullProgressMonitor());
      configureCCppProject(project);
   }
   
   
   private static void configureCCppProject(IProject project) 
      throws CoreException
   {    
      // Begin code based on org.eclipse.cdt.managedbuilder.ui.wizards.
      // StdBuildWizard.createProject(IProject, boolean, boolean) (and
      // methods called by it)
      
      //FYI: ICProjectDescription has nothing to do with IProjectDescription
      ICProjectDescriptionManager mngr = CoreModel.getDefault().
         getProjectDescriptionManager();
      
      ICProjectDescription des = mngr.createProjectDescription(project, 
            false,   //don't bother loading; it doesn't exist
            false    //don't set isCdtProjectCreating flag
            );
      
      ManagedBuildInfo info = ManagedBuildManager.createBuildInfo(project);
      ManagedProject mProj = new ManagedProject(des);
      info.setManagedProject(mProj);

      //Get the available toolChains, and then locate a GCC tool chain 
      IToolChain[] toolChains = ManagedBuildManager.getRealToolChains();
      IToolChain toolChain = null;
      for (int i = 0; i < toolChains.length; i++) {
         // see org.eclipse.cdt.managedbuilder.ui.wizards.AbstractCWizard.isValid(IToolChain)
         // for the origin of the following tests
         if ( toolChains[i].getName().contains("GCC") &&
               isValid(toolChains[i]) )
         {
            toolChain = toolChains[i];
            break;
         }
      }
      //may leave toolChain null if none found
      CfgHolder cfgHolder = new CfgHolder(toolChain, null);
               
      String s = (toolChain == null) ? "0" : ((ToolChain)toolChain).getId();  //$NON-NLS-1$
      Configuration cfg = new Configuration(mProj, (ToolChain)toolChain, 
            ManagedBuildManager.calculateChildId(s, null), 
            cfgHolder.getName());

      IBuilder bld = cfg.getEditableBuilder();
      if (bld != null) {
         if(bld.isInternalBuilder()){
            IConfiguration prefCfg = ManagedBuildManager.getPreferenceConfiguration(false);
            IBuilder prefBuilder = prefCfg.getBuilder();
            cfg.changeBuilder(prefBuilder, ManagedBuildManager.calculateChildId(cfg.getId(), null), prefBuilder.getName());
            bld = cfg.getEditableBuilder();
            bld.setBuildPath(null);
         }
         bld.setManagedBuildOn(false);
      } else {
         System.out.println("IBuilder is null"); //$NON-NLS-1$
      }
      cfg.setArtifactName(CWizardHandler.removeSpaces(project.getName()));
      CConfigurationData data = cfg.getConfigurationData();
      des.createConfiguration(ManagedBuildManager.CFG_DATA_PROVIDER_ID, data);

      mngr.setProjectDescription(project, des);
            
      // End code based on StdBuildWizard.createProject
   }
   

   /**
    * Is the given toolchain valid on this platform?
    * @param tc
    * @return
    */
   private static boolean isValid(IToolChain tc) {

      // see org.eclipse.cdt.managedbuilder.ui.wizards.AbstractCWizard.isValid(IToolChain)
      // for the origin of the following tests
      if ( !tc.isSupported() ||
            tc.isAbstract() ||
            tc.isSystemObject() ) return false;
         
      //CDT_5_0_2 has the following method defined, but
      //CDT_4_0_2 does not.  Try the v5 method, and then v4
      //I am commenting this out because it is better to develop
      //with Eclipse 3.3 to ensure compatibility and this won't
      //compile against 3.3.
//      try {
//         if ( ManagedBuildManager.isPlatformOk(tc) ) return true;
//         else return false;
//      }
//      catch( NoSuchMethodError nsme ) {}

      //Use the v4 code
      final String ALL = "all";
      String os = Platform.getOS();
      String arch = Platform.getOSArch();
      
      ITargetPlatform tp = tc.getTargetPlatform();
      if (tp != null) {
         List<String> osList = Arrays.asList(tc.getOSList());
         if (osList.contains(ALL) || osList.contains(os)) {
            List<String> archList = Arrays.asList(tc.getArchList());
            if (archList.contains(ALL) || archList.contains(arch))
               return true; // OS and ARCH fits
         }
         return false; // OS or ARCH does not fit
      }
      return true; // No platform: nothing to check       
   }
   
   
   public static void configureCppProject(IProject project) throws CoreException
   {  
      CProjectNature.addCNature(project, new NullProgressMonitor());
      CCProjectNature.addCCNature(project, new NullProgressMonitor());
      configureCCppProject(project);
   }
}
