package com.objsys.asn1c.validator.lib;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.ClasspathContainerInitializer;
import org.eclipse.jdt.core.IAccessRule;
import org.eclipse.jdt.core.IClasspathAttribute;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.osgi.framework.Bundle;

import com.objsys.asn1c.Activator;


public class ASN1CClasspathContainerInitializer extends
  org.eclipse.jdt.core.ClasspathContainerInitializer {

  private static final String LIBS_PATH_ASN1RT_JAR = "/libs/asn1rt.jar"; //$NON-NLS-1$
  private static final String LIBS_PATH_ASN1RT_JAVADOC_JAR = "/libs/asn1rt_javadoc.jar"; //$NON-NLS-1$

  private static class ASN1CClasspathContainer implements IClasspathContainer {

    private final IClasspathEntry[] fEntries;
    private final IPath fPath;

    public ASN1CClasspathContainer(IPath path, IClasspathEntry[] entries) {
      fPath= path;
      fEntries= entries;
    }

    public IClasspathEntry[] getClasspathEntries() {
      return fEntries;
    }

    public String getDescription() {
      return Messages.ASN1CClasspathContainer_name;
    }

    public int getKind() {
      return IClasspathContainer.K_APPLICATION;
    }

    public IPath getPath() {
      return fPath;
    }

  }
  private static final IStatus NOT_SUPPORTED= new Status(IStatus.ERROR, Activator.PLUGIN_ID, ClasspathContainerInitializer.ATTRIBUTE_NOT_SUPPORTED, new String(), null);
  private static final IStatus READ_ONLY= new Status(IStatus.ERROR, Activator.PLUGIN_ID, ClasspathContainerInitializer.ATTRIBUTE_READ_ONLY, new String(), null);

  public static final String RT = "RT"; //$NON-NLS-1$
  public static final String ASN1C_LIBS_ID = "asn1.ext.validator.ASN1C_LIBS"; //$NON-NLS-1$
  public final static IPath ASN1C_LIBS_PATH= new Path(ASN1C_LIBS_ID).append(RT);

  public ASN1CClasspathContainerInitializer() {
  }

  @Override
  public void initialize(IPath containerPath, IJavaProject project) throws CoreException {
    if (isASN1CLibraryContainer(containerPath)) {
      ASN1CClasspathContainer container= getNewContainer(containerPath);
      JavaCore.setClasspathContainer(containerPath,
          new IJavaProject[] { project },
          new IClasspathContainer[] { container }, null);
    }
  }

  private static ASN1CClasspathContainer getNewContainer(IPath containerPath) {
    IClasspathEntry[] entries= null;
    String version= containerPath.segment(1);
    if (RT.equals(version)) {
      entries= getClasspathEntries();
    }

    if (entries == null) {
      entries= new IClasspathEntry[] { };
    }
    return new ASN1CClasspathContainer(containerPath, entries);
  }

  /**
   * Variable to return an empty array of <code>IAccessRule</code>s
   */
  private static IAccessRule[] EMPTY_RULES = new IAccessRule[0];

  public static IClasspathEntry[] getClasspathEntries() {
    IPath jarPath = getASN1RTLib();
    if (jarPath != null) {
      IClasspathAttribute[] attributes = null;
      IPath javadocLocation = getASN1RTLibJavaDoc();
      String javadocUrl = null;
      try {
        javadocUrl = "jar:"+javadocLocation.toFile().toURL().toExternalForm()+"!/"; //$NON-NLS-1$ //$NON-NLS-2$
      } catch (Exception e) {
        javadocUrl = null;
      }
      if (javadocUrl == null) {
        attributes = new IClasspathAttribute[0];
      } else {
        attributes = new IClasspathAttribute[]{JavaCore.newClasspathAttribute(IClasspathAttribute.JAVADOC_LOCATION_ATTRIBUTE_NAME, javadocUrl)};
      }
      IClasspathEntry entry = JavaCore.newLibraryEntry(jarPath, null, null, EMPTY_RULES, attributes, false);
      IClasspathEntry[] result = new IClasspathEntry[] { entry };
      return result;
    }
    return null;
  }
  public static IPath getASN1RTLibJavaDoc() {
    try {
      Bundle bundle = Activator.getDefault().getBundle();
      IPath asnrtJavaDocLocation = getBundleLocation(bundle, new Path(LIBS_PATH_ASN1RT_JAVADOC_JAR));
      return asnrtJavaDocLocation;
    } catch (Exception e) {
      return null;
    }
  }
  public static IPath getASN1RTLib() {
    try {
      Bundle bundle = Activator.getDefault().getBundle();
      IPath asnrtLocation = getBundleLocation(bundle, new Path(LIBS_PATH_ASN1RT_JAR));
      return asnrtLocation;
    } catch (Exception e) {
      return null;
    }
  }
  private boolean isASN1CLibraryContainer(IPath path) {
    return path != null && path.segmentCount() == 2 && ASN1C_LIBS_ID.equals(path.segment(0));
  }


  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#canUpdateClasspathContainer(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject)
   */
  public boolean canUpdateClasspathContainer(IPath containerPath, IJavaProject project) {
    return true;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getAccessRulesStatus(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject)
   */
  public IStatus getAccessRulesStatus(IPath containerPath, IJavaProject project) {
    return NOT_SUPPORTED;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getSourceAttachmentStatus(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject)
   */
  public IStatus getSourceAttachmentStatus(IPath containerPath, IJavaProject project) {
    return READ_ONLY;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getAttributeStatus(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject, java.lang.String)
   */
  public IStatus getAttributeStatus(IPath containerPath, IJavaProject project, String attributeKey) {
    if (attributeKey.equals(IClasspathAttribute.JAVADOC_LOCATION_ATTRIBUTE_NAME)) {
      return Status.OK_STATUS;
    }
    return NOT_SUPPORTED;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#requestClasspathContainerUpdate(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject, org.eclipse.jdt.core.IClasspathContainer)
   */
  public void requestClasspathContainerUpdate(IPath containerPath, IJavaProject project, IClasspathContainer containerSuggestion) throws CoreException {
    IClasspathEntry[] entries= containerSuggestion.getClasspathEntries();
    if (entries.length == 1 && isASN1CLibraryContainer(containerPath)) {
      rebindClasspathEntries(project.getJavaModel(), containerPath);
    }
  }

  private static void rebindClasspathEntries(IJavaModel model, IPath containerPath) throws JavaModelException {
    ArrayList<IJavaProject> affectedProjects= new ArrayList<IJavaProject>();

    IJavaProject[] projects= model.getJavaProjects();
    for (int i= 0; i < projects.length; i++) {
      IJavaProject project= projects[i];
      IClasspathEntry[] entries= project.getRawClasspath();
      for (int k= 0; k < entries.length; k++) {
        IClasspathEntry curr= entries[k];
        if (curr.getEntryKind() == IClasspathEntry.CPE_CONTAINER && containerPath.equals(curr.getPath())) {
          affectedProjects.add(project);
        }
      }
    }
    if (!affectedProjects.isEmpty()) {
      IJavaProject[] affected= (IJavaProject[]) affectedProjects.toArray(new IJavaProject[affectedProjects.size()]);
      IClasspathContainer[] containers= new IClasspathContainer[affected.length];
      for (int i= 0; i < containers.length; i++) {
        containers[i]= getNewContainer(containerPath);
      }
      JavaCore.setClasspathContainer(containerPath, affected, containers, null);
    }
  }

  /**
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getDescription(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject)
   */
  public String getDescription(IPath containerPath, IJavaProject project) {
    if (isASN1CLibraryContainer(containerPath)) {
      String version= containerPath.segment(1);
      if (RT.equals(version)) {
        return Messages.ASN1CClasspathContainer_description;
      }
    }
    return Messages.ASN1CClasspathContainer_description_initializer_unresolved;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jdt.core.ClasspathContainerInitializer#getComparisonID(org.eclipse.core.runtime.IPath, org.eclipse.jdt.core.IJavaProject)
   */
  public Object getComparisonID(IPath containerPath, IJavaProject project) {
    return containerPath;
  }

  public static IPath getBundleLocation(Bundle bundle, Path path) {
    if (bundle == null)
      return null;

    URL local= null;
    try {
      URL[] findEntries = FileLocator.findEntries(bundle, path);
      local= FileLocator.toFileURL(findEntries[0]);
      String fullPath = new File(local.toURI()).getAbsolutePath();
      return Path.fromOSString(fullPath);
    } catch (Exception e) {
      return null;
    }
  }
}
