package com.objsys.asn1c.validator.project;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;

import com.objsys.asn1c.Activator;


public class ASN1CNature implements IProjectNature {
	public static final String ASN1C_NATURE_ID = "asn1.ext.asn1c.nature"; //$NON-NLS-1$

  private IProject project;

  /* (non-Javadoc)
   * @see org.eclipse.core.resources.IProjectNature#configure()
   */
  public void configure() throws CoreException {
  }

  /* (non-Javadoc)
   * @see org.eclipse.core.resources.IProjectNature#deconfigure()
   */
  public void deconfigure() throws CoreException {
  }

  /* (non-Javadoc)
   * @see org.eclipse.core.resources.IProjectNature#getProject()
   */
  public IProject getProject() {
    return project;
  }

  /* (non-Javadoc)
   * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
   */
  public void setProject(IProject project) {
    this.project = project;
  }

  /**
   * @param project
   */
  public static synchronized void toggleNature(final IProject project) {
  	if (project == null)
  		return;

  	try {
  		final boolean hasAsnNature = ASN1CNature.hasNature(project);
  		if (hasAsnNature) {
  		  ASN1CNature.removeNature(project);
  		} else {
  		  ASN1CNature.addNature(project);
  		}
  	} catch (CoreException e) {
  	  Activator.log(e);
  	}
  }

  public static synchronized boolean hasNature(IProject project) throws CoreException {
  		return project.hasNature(ASN1CNature.ASN1C_NATURE_ID);
  }

  public static synchronized void addNature(IProject project) throws CoreException {
  	if ((project == null) || hasNature(project))
  		return;

  	IProjectDescription description = project.getDescription();
  	String[] natures = description.getNatureIds();
  	String[] newNatures = new String[natures.length + 1];
  	System.arraycopy(natures, 0, newNatures, 1, natures.length);
  	newNatures[0] = ASN1CNature.ASN1C_NATURE_ID;

    // check the status and decide what to do
  	IStatus status = project.getWorkspace().validateNatureSet(newNatures);
    if (status.getCode() == IStatus.OK) {
    	description.setNatureIds(newNatures);
    	project.setDescription(description, null);
    } else {
    	// raise a user error
    	Activator.log(status);
    }
  }

  public static synchronized void removeNature(IProject project) throws CoreException {
  	if ((project == null) || !hasNature(project))
  		return;

  	IProjectDescription description = project.getDescription();
  	String[] natures = description.getNatureIds();
  	String[] newNatures = new String[natures.length - 1];
  	for (int i = 0; i < natures.length; i++) {
  		if (ASN1CNature.ASN1C_NATURE_ID.equals(natures[i])) {
  			System.arraycopy(natures, 0, newNatures, 0, i);
  			System.arraycopy(natures, i + 1, newNatures, i, natures.length - i - 1);

  	    // check the status and decide what to do
  	  	IStatus status = project.getWorkspace().validateNatureSet(newNatures);
  	    if (status.getCode() == IStatus.OK) {
  	    	description.setNatureIds(newNatures);
  	    	project.setDescription(description, null);
  	    } else {
  	    	// raise a user error
  	      Activator.log(status);
  	    }
  			break;
  		}
  	}
  }

}
