package com.objsys.asn1c.validator.wizard;

import net.sourceforge.asneditor.ui.preferences.common.AbstractConfigurationBlock;
import net.sourceforge.asneditor.ui.preferences.common.ScrolledPageContent;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

import com.objsys.asn1c.validator.ValidationUtils;
import com.objsys.asn1c.validator.preferences.ASN1CConfigurationBlock;
import com.objsys.asn1c.validator.project.ACProject;

/**
 * The "New" wizard page allows setting the container for the new file as well
 * as the file name. The page will only accept file name without the extension
 * OR with the extension that matches the expected one (acp, acpproj).
 */

public class ASN1CProjectWizardPage extends WizardPage {

  private final ASN1CConfigurationBlock configurationBlock;
  private Text cmdLineText;
  private final ACProject acproject;
  private IPropertyChangeListener propertyChangeListener;

	/**
	 * Constructor for SampleNewWizardPage.
	 * @param acproject
	 * @param acproject
	 * @param configurationBlock
	 *
	 * @param pageName
	 */
	public ASN1CProjectWizardPage(ACProject acproject, ASN1CConfigurationBlock configurationBlock) {
		super("acpWizardPage"); //$NON-NLS-1$
    this.acproject = acproject;
    this.configurationBlock = configurationBlock;
		setTitle(Messages.ProjectWizardPage_title);
		setDescription(Messages.ProjectWizardPage_description);
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
    ScrolledPageContent scrolled= new ScrolledPageContent(parent);
    scrolled.setExpandHorizontal(true);
    scrolled.setExpandVertical(true);

    Composite rootComposite= scrolled.getBody();
    GridLayout layout= new GridLayout();
    layout.marginWidth= 0;
    layout.marginHeight= 0;
    rootComposite.setLayout(layout);
    scrolled.setContent(rootComposite);

    Group group = new Group(rootComposite, SWT.NONE);
    group.setText(com.objsys.asn1c.validator.preferences.Messages.ASN1CConfigurationBlock_general);
    group.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
    AbstractConfigurationBlock.applyGridLayout(1, group);
	  configurationBlock.addGeneralSection(group, false);

    cmdLineText = new Text(group, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    cmdLineText.setEditable(false);
    GridData gd = new GridData(GridData.FILL, GridData.FILL, true, true);
    gd.minimumHeight = 150;
    cmdLineText.setLayoutData(gd);
    propertyChangeListener = new IPropertyChangeListener(){
      public void propertyChange(PropertyChangeEvent event) {
        dialogChanged();
      }};
    acproject.getPreferenceStore().addPropertyChangeListener(propertyChangeListener);

    // initialize fields only after their creation
    configurationBlock.initialize();
    dialogChanged();
    setControl(scrolled);
	}
  private void dialogChanged() {
    if (cmdLineText == null || cmdLineText.isDisposed() || acproject == null) {
      return;
    }
    cmdLineText.setText(ValidationUtils.getCommandLineParameters(acproject, null, 100));
  }

  public void dispose() {
    if (propertyChangeListener != null)
      acproject.getPreferenceStore().removePropertyChangeListener(propertyChangeListener);
    super.dispose();
  }
}