package com.objsys.asn1c.validator.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import net.sourceforge.asneditor.ASNNature;
import net.sourceforge.asneditor.util.ProjectUtils;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.ui.wizards.NewJavaProjectWizardPage;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;

import com.objsys.asn1c.Activator;
import com.objsys.asn1c.cdt.Utility;
import com.objsys.asn1c.validator.ValidationUtils;
import com.objsys.asn1c.validator.preferences.ASN1CConfigurationBlock;
import com.objsys.asn1c.validator.preferences.ASN1CPreferenceConstants;
import com.objsys.asn1c.validator.project.ACProject;
import com.objsys.asn1c.validator.project.ASN1CNature;

public class ASN1CWizard extends Wizard implements INewWizard {

  private ACProject acproject;

  private ASN1CConfigurationBlock configurationBlock;

  private WizardNewProjectCreationPage mainPage; // new proj

  private ASN1CProjectWizardPage projPage; // edit proj

  private FilesWizardPage filesPage;
  //private LanguageWizardPage langPage;

  private NewJavaProjectWizardPage javaPrjPage; // new proj

  //private FunctionsWizardPage functionsPage;
  //private BasicWizardPage basicPage;
  //private JavaWizardPage javaPage;
  private CppWizardPage cppPage;
  private CSharpWizardPage cSharpPage;
  private XSDWizardPage xsdPage;
  private LaunchWizardPage launchPage;

  private IWorkbench workbench;

  private final boolean newProject;

  private LinkedList<WizardPageElement> pages;

  /**
   * Combines an IWizardPage with an enabled flag.
   * Used for tracking the wizard pages and whether they should be enabled
   * or not.
   */
  private class WizardPageElement {
    IWizardPage page;
    boolean enabled;
    public WizardPageElement(IWizardPage page) {
      this.page = page;
      enabled = true;
    }
    public void enable() {
      enabled = true;
    }
    public void disable() {
      enabled = false;
    }
  }

  /**
   * @param newProject if <b>true</b> a new project wizard is created,
   * else an editing wizard is opened on an existing project
   */
  public ASN1CWizard(boolean newProject) {
    super();
    
    setNeedsProgressMonitor(true);
    ImageDescriptor image =
      Activator.imageDescriptorFromPlugin(
        Activator.PLUGIN_ID, "icons/wizbanner.png"); //$NON-NLS-1$
    setDefaultPageImageDescriptor(image);
    this.newProject = newProject;
  }

  public ASN1CWizard() {
    this(true);
  }

  public void addPages() {
    pages = new LinkedList<WizardPageElement>();
    configurationBlock = new ASN1CConfigurationBlock(acproject.getPreferenceStore());

    if (newProject) {
      mainPage = new WizardNewProjectCreationPage("newACProjectPage"); //$NON-NLS-1$
      mainPage.setTitle(Messages.ProjectWizardPage_title);
      mainPage.setDescription(Messages.ProjectWizardPage_description);
      addPage(mainPage);
    } else {
      // edit existing project - show configuration
      projPage = new ASN1CProjectWizardPage(acproject, configurationBlock);
      addPage(projPage);
    }
    filesPage = new FilesWizardPage(acproject, configurationBlock);
    addPage(filesPage);
   // langPage = new LanguageWizardPage(acproject, configurationBlock);
    //addPage(langPage);

    if (newProject) {
       javaPrjPage = new NewJavaProjectWizardPage(ResourcesPlugin.getWorkspace().getRoot(), mainPage){
        private boolean initialized;
        public void setVisible(boolean visible) {
          if (!initialized) { // only once
            if (visible) {
              // initialize the source path first
              super.setVisible(visible);
              // append default class path
              IClasspathEntry[] defaultClasspathEntries = ACProject.getDefaultClasspathEntries();
              Set<IClasspathEntry> entries = new HashSet<IClasspathEntry>(Arrays.asList(defaultClasspathEntries));
              entries.addAll(Arrays.asList(javaPrjPage.getRawClassPath()));
              javaPrjPage.setDefaultClassPath(entries.toArray(new IClasspathEntry[0]), false);
              initBuildPaths();
              return;
            } else {
              initialized = true;
              copyJavaSrcDir();
            }
          }
          super.setVisible(visible);
        }
      };
      addPage(javaPrjPage);
    }
    //functionsPage = new FunctionsWizardPage(acproject, configurationBlock);
    //addPage(functionsPage);
   // basicPage = new BasicWizardPage(acproject, configurationBlock);
  //  addPage(basicPage);
   // javaPage = new JavaWizardPage(acproject, configurationBlock);
   // addPage(javaPage);
    cppPage = new CppWizardPage(acproject, configurationBlock);
    addPage(cppPage);
    cSharpPage = new CSharpWizardPage(acproject, configurationBlock);
    addPage(cSharpPage);
    xsdPage = new XSDWizardPage(acproject, configurationBlock);
    addPage(xsdPage);
    launchPage = new LaunchWizardPage(acproject);
    addPage(launchPage);

    boolean enableXSD = acproject.getPreferenceStore().getBoolean(ASN1CPreferenceConstants.XSD);
    String language = acproject.getPreferenceStore().getString(ASN1CPreferenceConstants.LANGUAGE);
    changeLanguage(enableXSD, language);
  }

  @Override
  public void addPage(IWizardPage page) {
    super.addPage(page);
    pages.add(new WizardPageElement(page));
  }

  @Override
  public boolean performFinish() {
    if (newProject) {
      boolean javaPageEnabled = isWizardPageElementEnabled(javaPrjPage);

      // create a new project
      final IRunnableWithProgress runnable;
      if (javaPageEnabled) {
        runnable = javaPrjPage.getRunnable();
      } else {
        runnable = getRunnable();
      }
      IRunnableWithProgress op= new WorkspaceModifyDelegatingOperation(runnable);
      try {
        getContainer().run(false, true, op);
        IJavaProject newJavaProject = null;
        if (javaPageEnabled) {
          newJavaProject = javaPrjPage.getNewJavaProject();
        }
        IProject project = mainPage.getProjectHandle();
        
        String language = acproject.getPreferenceStore().getString(
              ASN1CPreferenceConstants.LANGUAGE);
        boolean langC = ASN1CPreferenceConstants.LANG_C.equals(language);
        boolean langCpp = !langC && 
           ASN1CPreferenceConstants.LANG_CPP.equals(language);
        
        if ( langC || langCpp ) {
           try {
              if ( langC )
                 Utility.configureCProject(project);
              else
                 Utility.configureCppProject(project);
           }
           catch (NoClassDefFoundError noclass) {
              ErrorDialog.openError(this.getShell(), 
                    "Error", 
                    "Unable to create C/C++ project.\n"+
                    "Have you installed the CDT support plugin?",
                    new Status(Status.ERROR, Activator.PLUGIN_ID, 
                          "Unable to create C/C++ project", noclass));              
           }        
        }

        // add ASN1C settings
        acproject.setProject(project);
        ASN1CNature.addNature(project);

        if (javaPageEnabled) {
          ACProject.addJarsToClassPath(newJavaProject);
        }

        // add ASN.1 Editor settings
        ASNNature.addAsnNature(project);
        LinkedList<IResource> linkedList = new LinkedList<IResource>();
        linkedList.add(project.getFolder(acproject.getSchemaPath()));
        ProjectUtils.saveImportPath(project, linkedList);

      } catch (Exception e) {
        Activator.log(Messages.ASN1CWizard_Error_while_creating_project, e);
        return false;
      }

      selectAndReveal(acproject.getProject());
    } else {
      // edit selected project
    }
    // set output folder or if not absolute path
    String output = acproject.getPreferenceStore().getString(ASN1CPreferenceConstants.OUTPUT_DIR);
    if (output == null || output.trim().length() == 0) {
      output = "/"+acproject.getProject().getName()+"/src"; //$NON-NLS-1$ //$NON-NLS-2$
      acproject.getPreferenceStore().setValue(ASN1CPreferenceConstants.OUTPUT_DIR, output);
    } else {
      Path path = new Path(output);
      if (!path.isAbsolute()) {
        output = "/"+acproject.getProject().getName() + '/' + output; //$NON-NLS-1$
        acproject.getPreferenceStore().setValue(ASN1CPreferenceConstants.OUTPUT_DIR, output);
      }
    }

    try {
      acproject.copySchemaFilesToWorkspace();
      acproject.save();

      // create src folder
      Path path = new Path(output);
      IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
      if (!root.exists(path)) {
        root.getFolder(path).create(true, true, new NullProgressMonitor());
      }
    } catch (Exception ex) {
      String msg = NLS.bind(Messages.LaunchWizardPage_project_save_error, ex.getMessage());
      MessageDialog.openError(getShell(), Messages.LaunchWizardPage_Error, msg);
      Activator.log(msg, ex);
    }

    if (launchPage.shouldGenerate()) {
      ValidationUtils.generate(acproject.getProject());
    }
    return true;
  }

  public IRunnableWithProgress getRunnable() {
    return new IRunnableWithProgress() {
      public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
        if (monitor == null) {
          monitor= new NullProgressMonitor();
        }
        monitor.beginTask(Messages.create_project, 10);

        // create the project
        try {
          IProject projectHandle = mainPage.getProjectHandle();
          projectHandle.create(monitor);
          projectHandle.open(monitor);
          ACProject.create(projectHandle);
        } catch (CoreException e) {
          throw new InvocationTargetException(e);
        } catch (OperationCanceledException e) {
          throw new InterruptedException();
        } finally {
          monitor.done();
        }
      }
    };
  }

/* KTB: no references to this method exist?
 
  public static void createProject(IProject project, URI locationURI, IProgressMonitor monitor) throws CoreException {
    if (monitor == null) {
      monitor= new NullProgressMonitor();
    }
    monitor.beginTask(Messages.create_project, 10);

    // create the project
    try {
      if (!project.exists()) {
        IProjectDescription desc= project.getWorkspace().newProjectDescription(project.getName());
        if (locationURI != null && ResourcesPlugin.getWorkspace().getRoot().getLocationURI().equals(locationURI)) {
          locationURI= null;
        }
        desc.setLocationURI(locationURI);
        project.create(desc, monitor);
        monitor= null;
      }
      if (!project.isOpen()) {
        project.open(monitor);
        monitor= null;
      }
    } finally {
      if (monitor != null) {
        monitor.done();
      }
    }
  }
*/

  public void init(IWorkbench workbench, IStructuredSelection selection) {
    this.workbench = workbench;

    IProject project = null;
    if (!newProject && selection != null && selection.isEmpty() == false
        && selection instanceof IStructuredSelection) {
      IStructuredSelection ssel = (IStructuredSelection) selection;
      if (ssel.size() == 1) {
        Object obj = ssel.getFirstElement();
        if (obj instanceof IResource) {
          project = ((IResource) obj).getProject();
        }
      }
    }
    acproject = new ACProject(project);
    if (newProject) {
      // set some defaults
      acproject.getPreferenceStore().setValue(ASN1CPreferenceConstants.OUTPUT_DIR, "src"); //$NON-NLS-1$
    } else {
      try {
        acproject.loadPreferenceStore();
      } catch (Exception ex) {
        // ignore these
      }
    }
    // Create a property change listener on the preference store so that
    // we can invoke changeLanguage on any preference change.
    acproject.getPreferenceStore().addPropertyChangeListener(new IPropertyChangeListener(){
      public void propertyChange(PropertyChangeEvent event) {
        String key = event.getProperty();
        if (ASN1CPreferenceConstants.XSD.equals(key)) {
          boolean enableXSD = Boolean.parseBoolean(""+event.getNewValue()); //$NON-NLS-1$
          String language = acproject.getPreferenceStore().getString(ASN1CPreferenceConstants.LANGUAGE);
          changeLanguage(enableXSD, language);
        } else if (ASN1CPreferenceConstants.LANGUAGE.equals(key)) {
          boolean enableXSD = acproject.getPreferenceStore().getBoolean(ASN1CPreferenceConstants.XSD);
          String language = ""+event.getNewValue(); //$NON-NLS-1$
          changeLanguage(enableXSD, language);
        }
      }});
  }

  /**
   * Flag a page as enabled/disabled by looking up the associated 
   * WizardPageElement in {@code pages} and setting its enabled flag.
   * 
   * @param page
   * @param enabled
   */
  protected void setWizardPageElement(IWizardPage page, boolean enabled) {
    WizardPageElement wizardPageElement = getWizardPageElement(page);
    if (wizardPageElement != null) {
      wizardPageElement.enabled = enabled;
    }
  }

  protected boolean isWizardPageElementEnabled(IWizardPage page) {
    WizardPageElement wizardPageElement = getWizardPageElement(page);
    if (wizardPageElement != null) {
      return wizardPageElement.enabled;
    }
    return false;
  }
  public IWizardPage getNextPage(IWizardPage currentPage) {
    if (newProject && currentPage instanceof WizardNewProjectCreationPage) {
      acproject.setProject(mainPage.getProjectHandle());
//    } else if (newProject && currentPage instanceof NewJavaProjectWizardPage) {
//      copyJavaSrcDir();
    }

    WizardPageElement page = getWizardPageElement(currentPage);
    if (page == null) {
      return null;
    }

    return getNeighborPage(+1, page); // getNext
  }

  public IWizardPage getPreviousPage(IWizardPage currentPage) {

    WizardPageElement page = getWizardPageElement(currentPage);
    if (page == null) {
      return null;
    }

    return getNeighborPage(-1, page); // getPrev
  }

  /**
   * @param inc if +1 is getNextPage, if -1 getPrevPage
   * @param page
   * @return
   */
  private IWizardPage getNeighborPage(int inc, WizardPageElement page) {
    int index = pages.indexOf(page);
    WizardPageElement nextPage = null;
    while (pages.size() > (index = index + inc) && index >= 0) {
      page = pages.get(index);
      if (page.enabled) {
        nextPage = page;
        break;
      }
    }
    if (nextPage != null) {
      return nextPage.page;
    }
    return null;
  }

  /**
   * Locate the WizardPageElement in {@code pages} associated with the given
   * wizard page.
   * 
   * @param currentPage
   * @return
   */
  private WizardPageElement getWizardPageElement(IWizardPage currentPage) {
    Iterator<WizardPageElement> it = pages.iterator();
    WizardPageElement page;
    while (it.hasNext()) {
      page = it.next();
      if (page.page == currentPage) {
        return page;
      }
    }
    return null;
  }

  protected void copyJavaSrcDir() {
    try {
      IClasspathEntry[] rawClasspath = javaPrjPage.getRawClassPath();
      for (IClasspathEntry classpathEntry : rawClasspath) {
        if (classpathEntry.getEntryKind() == IClasspathEntry.CPE_SOURCE) {
          IPath sourcePath = classpathEntry.getPath();
          IFolder folder = ResourcesPlugin.getWorkspace().getRoot().getFolder(sourcePath);
          String outputDir = acproject.getPreferenceStore().getString(ASN1CPreferenceConstants.OUTPUT_DIR);
          if (outputDir == null || outputDir.trim().length() == 0) {
            acproject.getPreferenceStore().setValue(ASN1CPreferenceConstants.OUTPUT_DIR, folder.getProjectRelativePath().toPortableString());
            break;
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
      // TODO: handle exception
    }
  }

  public void dispose() {
    configurationBlock.dispose();
    if (acproject != null) {
      acproject.getPreferenceStore().stop();
      acproject = null;
    }
    super.dispose();
  }

  protected void selectAndReveal(IResource newResource) {
    BasicNewResourceWizard.selectAndReveal(newResource, workbench.getActiveWorkbenchWindow());
  }

  protected void changeLanguage(boolean enableXSD, String language) {
    if (ASN1CPreferenceConstants.LANG_C.equals(language)) {
      enablePages(true, false, false, enableXSD);
    } else if (ASN1CPreferenceConstants.LANG_CPP.equals(language)) {
      enablePages(true, false, false, enableXSD);
    } else if (ASN1CPreferenceConstants.LANG_C_SHARP.equals(language)) {
      enablePages(false, true, false, enableXSD);
    } else if (ASN1CPreferenceConstants.LANG_JAVA.equals(language)) {
      enablePages(false, false, true, enableXSD);
    } else {
      enablePages(false, false, false, false);
    }
  }

  /**
   * Enable wizard pages only if the given flag is set
   * @param cpp
   * @param csharp
   * @param java
   * @param xsd
   */
  protected void enablePages(boolean cpp, boolean csharp, boolean java, boolean xsd) {
    final boolean generation = cpp | csharp | java;

    //setWizardPageElement(basicPage, generation);
    setWizardPageElement(cppPage, cpp);
    setWizardPageElement(cSharpPage, csharp);
   // setWizardPageElement(javaPage, java);
    setWizardPageElement(javaPrjPage, java);
    setWizardPageElement(xsdPage, generation && xsd);
    //setWizardPageElement(functionsPage, generation);

    configurationBlock.enableKeys(cpp, csharp, java, xsd);
  }
}
