package com.objsys.asn1c.validator.wizard;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import net.sourceforge.asneditor.ui.dialogfields.IListAdapter;
import net.sourceforge.asneditor.ui.dialogfields.ListDialogField;
import net.sourceforge.asneditor.ui.preferences.common.AbstractConfigurationBlock;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

import com.objsys.asn1c.validator.preferences.ASN1CConfigurationBlock;
import com.objsys.asn1c.validator.preferences.ASN1CPreferenceConstants;
import com.objsys.asn1c.validator.project.ACProject;


public class FilesWizardPage extends WizardPage {
  private final ACProject acproject;
  private ASN1CConfigurationBlock configurationBlock;
  private ListDialogField schemaFiles;
  private ListDialogField importDirs;
  private boolean initialized;

  protected FilesWizardPage(ACProject acproject, ASN1CConfigurationBlock configurationBlock) {
    super("FilesWizardPage"); //$NON-NLS-1$
    this.acproject = acproject;
    this.configurationBlock = configurationBlock;
    setTitle(Messages.FilesWizardPage_title);
    setDescription(Messages.FilesWizardPage_description);
  }

  public void createControl(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    GridData gd = new GridData(GridData.FILL, GridData.FILL, true, true);
    composite.setLayoutData(gd);
    composite.setLayout(new GridLayout(2, false));

    IListAdapter schemaAdapter= new IListAdapter() {
      public void customButtonPressed(ListDialogField field, int index) {
        final String key = ASN1CPreferenceConstants.INPUT_FILES;

        switch (index) {
        case 0: { // add workspace resource
          IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
          ResourceSelectionDialog resDialog = new ResourceSelectionDialog(
              Display.getCurrent().getActiveShell(), root, Messages.SchemaListDialog_label);
          LinkedList<IFile> initialSel = new LinkedList<IFile>();
          Collection<String> collection = acproject.getCollection(key, ACProject.PATH_SEPARATOR);
          Collection<String> keepExternalFiles = new LinkedList<String>();
          for (String string : collection) {
            IFile file = root.getFile(new Path(string));
            if (file != null && file.exists()) {
              initialSel.add(file);
            } else {
              keepExternalFiles.add(string);
            }
          }
          resDialog.setInitialElementSelections(initialSel);
          if (resDialog.open() == Window.OK) {
            LinkedList<String> resultPaths = new LinkedList<String>();
            Object[] result = resDialog.getResult();
            for (Object object : result) {
              IFile file = (IFile) object;
              resultPaths.add(file.getFullPath().toPortableString());
            }
            resultPaths.addAll(keepExternalFiles);
            Collections.sort(resultPaths);
            acproject.setCollection(key, ACProject.PATH_SEPARATOR, resultPaths);
            field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
          }

        } break;
        case 1: { // copy file from external source
          FileDialog fileDialog = new FileDialog(Display.getCurrent().getActiveShell());
          fileDialog.setFilterExtensions(ASN1CPreferenceConstants.SCHEMA_EXT);
          fileDialog.setFilterNames(ASN1CPreferenceConstants.SCHEMA_EXT_NAMES);
          String selectedFile = fileDialog.open();
          if (selectedFile != null) {
            acproject.addToCollection(key, ACProject.PATH_SEPARATOR, selectedFile);
            LinkedList<String> resultPaths = new LinkedList<String>();
            resultPaths.addAll(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
            Collections.sort(resultPaths);
            field.setElements(resultPaths);
          }
        } break;
        case 2: {
          Collection<String> collection = acproject.getCollection(key, ACProject.PATH_SEPARATOR);
          collection.removeAll(field.getSelectedElements());
          acproject.setCollection(key, ACProject.PATH_SEPARATOR, collection);
          field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
        } break;
        case 3: {
          acproject.setCollection(key, ACProject.PATH_SEPARATOR, null);
          field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
        } break;
        }
      }
      public void selectionChanged(ListDialogField field) {
//        doSelectionChanged();
      }

      public void doubleClicked(ListDialogField field) {
//        doDoubleClicked();
      }
    };

    final String[] buttonLabels= new String[] {
        Messages.SchemaListDialog_select_resource_button,
        Messages.SchemaListDialog_add_file_button,
        Messages.SchemaListDialog_remove_button,
        Messages.SchemaListDialog_clear_button,
    };

    final String[] buttonTooltips= new String[] {
        Messages.SchemaListDialog_select_resource_button_desc,
        Messages.SchemaListDialog_add_file_button_desc,
        Messages.SchemaListDialog_remove_button_desc,
        Messages.SchemaListDialog_clear_button_desc,
    };

    schemaFiles = createList(Messages.SchemaListDialog_label, composite, schemaAdapter, buttonLabels, buttonTooltips);

    IListAdapter importAdapter= new IListAdapter() {
      public void customButtonPressed(ListDialogField field, int index) {
        final String key = ASN1CPreferenceConstants.IMPORT;

        switch (index) {
        case 0: {// add directory from workspace
          IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
          final IContainer initContainer = acproject.getProject() != null && acproject.getProject().isAccessible() ? acproject.getProject() : root;
          ContainerSelectionDialog dialog = new ContainerSelectionDialog(
              Display.getCurrent().getActiveShell(), initContainer, false, Messages.ProjectWizardPage_Select_file_container);

          LinkedList<IContainer> initialSel = new LinkedList<IContainer>();
          Collection<String> collection = acproject.getCollection(key, ACProject.PATH_SEPARATOR);
          Collection<String> keepExternalFiles = new LinkedList<String>();
          for (String string : collection) {
            IContainer container = root.getFolder(new Path(string));
            if (container != null && container.exists()) {
              initialSel.add(container);
            } else {
              keepExternalFiles.add(string);
            }
          }
          dialog.setInitialElementSelections(initialSel);
          if (dialog.open() == ContainerSelectionDialog.OK) {
            Object[] result = dialog.getResult();
            LinkedList<String> resultPaths = new LinkedList<String>();
            for (Object object : result) {
              Path path = (Path) object;
              resultPaths.add(path.toPortableString());
            }
            resultPaths.addAll(keepExternalFiles);
            Collections.sort(resultPaths);
            if (result.length == 1) {
              Path path = ((Path) result[0]);
              String string = path.toPortableString();
              acproject.addToCollection(key, ACProject.PATH_SEPARATOR, string);
              field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
            }
          }
        } break;
        case 1: {// add directory from file system
          DirectoryDialog dirDialog = new DirectoryDialog(Display.getCurrent().getActiveShell());
//          dirDialog.setFileName(string);
          String selectedFile = dirDialog.open();
          if (selectedFile != null) {
            acproject.addToCollection(key, ACProject.PATH_SEPARATOR, selectedFile);
            field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
          }
        } break;
        case 2: {// remove
          Collection<String> collection = acproject.getCollection(key, ACProject.PATH_SEPARATOR);
          collection.removeAll(field.getSelectedElements());
          acproject.setCollection(key, ACProject.PATH_SEPARATOR, collection);
          field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
        } break;
        case 3: {// clear
          acproject.setCollection(key, ACProject.PATH_SEPARATOR, null);
          field.setElements(acproject.getCollection(key, ACProject.PATH_SEPARATOR));
        } break;
        }
      }
      public void selectionChanged(ListDialogField field) {
//        doSelectionChanged();
      }

      public void doubleClicked(ListDialogField field) {
//        doDoubleClicked();
      }
    };

    final String[] importDirsButtonLabels= new String[] {
        Messages.SchemaListDialog_select_container_button,
        Messages.SchemaListDialog_add_folder_button,
        Messages.SchemaListDialog_remove_button,
        Messages.SchemaListDialog_clear_button,
    };

    final String[] importDirsButtonTooltips= new String[] {
        Messages.SchemaListDialog_select_container_button_desc,
        Messages.SchemaListDialog_add_folder_button_desc,
        Messages.SchemaListDialog_remove_button_desc,
        Messages.SchemaListDialog_clear_button_desc,
    };

    importDirs = createList(Messages.ImportListDialog_label, composite, importAdapter, importDirsButtonLabels, importDirsButtonTooltips);

    // select the first
    Collection<String> files = acproject.getCollection(ASN1CPreferenceConstants.INPUT_FILES, ACProject.PATH_SEPARATOR);
    schemaFiles.setElements(files);
    ISelection sel;
    if (files.isEmpty()) {
      sel= new StructuredSelection();
    } else {
      sel= new StructuredSelection(files.toArray()[0]);
    }
    schemaFiles.selectElements(sel);


    // select the first
    Collection<String> imports = acproject.getCollection(ASN1CPreferenceConstants.IMPORT, ACProject.PATH_SEPARATOR);
    importDirs.setElements(imports);
    if (imports.isEmpty()) {
      sel= new StructuredSelection();
    } else {
      sel= new StructuredSelection(imports.toArray()[0]);
    }
    importDirs.selectElements(sel);

    Composite local = AbstractConfigurationBlock.createComposite(composite, 1);
    GridData data = new GridData(GridData.FILL, GridData.BEGINNING, true, false);
    data.horizontalSpan = 2;
    local.setLayoutData(data);

    configurationBlock.addButtonOption(local, ASN1CPreferenceConstants.CONFIG, Messages.FilesWizardPage_Browse, configurationBlock.createFileListener());
    configurationBlock.addButtonOption(local, ASN1CPreferenceConstants.OUTPUT_DIR, Messages.FilesWizardPage_Browse, configurationBlock.createWorkspaceFolderListener(acproject.getProject(), Messages.ProjectWizardPage_Select_output_container));

    configurationBlock.addOption(local, ASN1CPreferenceConstants.LIST);
    configurationBlock.addOption(local, ASN1CPreferenceConstants.WARNINGS);

    // initialize fields only after their creation
    configurationBlock.initialize();

    setControl(composite);
  }

  private void getValues() {
    if (initialized) {
      return;
    }
    initialized = true;
//    ouputPathText.setText(acproject.getPreferenceStore().getString(ASN1CPreferenceConstants.OUTPUT_DIR));
  }

  private ListDialogField createList(String title, Composite composite, IListAdapter adapter, String[] buttonLabels, String[] buttonTooltips) {

    Label label = new Label(composite, SWT.NONE);
    label.setText(title);

    ListDialogField list= new ListDialogField(adapter, buttonLabels, buttonTooltips, new LabelProvider());
    list.setViewerComparator(new ViewerComparator());
    list.doFillIntoGrid(composite, 3);
    return list;
  }

  @Override
  public void setVisible(boolean visible) {
    super.setVisible(visible);

    if (visible) {
      getValues();
    }
  }
}
