/**
 * Copyright (c) 2005 Bogdan Stanca. All rights reserved.
 * 
 * This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * THIS SOFTWARE IS PROVIDED &quot;AS-IS,&quot;
 * WHITHOUT ANY EXPRESS OR IMPLIED WARRANTY.
 * 
 * In no event shall the author or authors be held liable
 * for any damages arising from the use of the software.
 * 
 */
package net.sourceforge.asneditor.editors;

import java.util.Iterator;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;

public class ASNTextHover implements ITextHover {

	ISourceViewer sourceViewer = null;
	
	public ASNTextHover(ISourceViewer sourceViewer) {
		this.sourceViewer = sourceViewer;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.ITextHover#getHoverInfo(org.eclipse.jface.text.ITextViewer, org.eclipse.jface.text.IRegion)
	 */
	public String getHoverInfo(ITextViewer textViewer, IRegion hoverRegion) {
		int offset = hoverRegion.getOffset();
		int length = hoverRegion.getLength();
		try {
			IAnnotationModel model = sourceViewer.getAnnotationModel();
			if (model != null) {
				Iterator e = model.getAnnotationIterator();
				while (e.hasNext()) {
					Annotation a = (Annotation)e.next();
						Position position = model.getPosition((Annotation)a);
						if (position.overlapsWith(offset, length)) {
							return a.getText();
					}
				}
			}
			// must be implemented
			return textViewer.getDocument().get(offset, length);
		}catch (Exception exc){
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.ITextHover#getHoverRegion(org.eclipse.jface.text.ITextViewer, int)
	 */
	public IRegion getHoverRegion(ITextViewer textViewer, int offset) {
		int start = offset + 1;
		int end = offset++;
		IDocument doc = textViewer.getDocument();
		int docEnd = doc.getLength();
		while (start >= 0) {
			// go to left to get begining of String
			try {
				if (!Character.isJavaIdentifierPart(doc.getChar(--start)))
					break;
			} catch (Exception exc) {
				break;
			}
		}
		start++;
		while (end < docEnd) {
			// go to left to get begining of String
			try {
				if (!Character.isJavaIdentifierPart(doc.getChar(end++)))
					break;
			} catch (Exception exc) {
				break;
			}
		}
		end --;

		return new Region(start, end - start);
	}

}
